#ifndef SNTP_H
#define SNTP_H

#include <stdint.h>
#include <sys/time.h>
#include <arpa/inet.h>

typedef struct {
    uint32_t second;
    uint32_t fraction;
} __attribute( ( packed ) ) SNTPTime;

typedef struct {
    uint8_t leapversionmode;
    uint8_t stratum;
    uint8_t poll;
    uint8_t precision;
    uint32_t rootdelay;
    uint32_t rootdispersion;
    uint32_t refid;
    SNTPTime ref_timestamp;
    SNTPTime orig_timestamp;
    SNTPTime rx_timestamp;
    SNTPTime tx_timestamp;
} __attribute( ( packed ) ) SNTPMessage;

//define the seconds from jan 1900 to 1970  2208988800
#define SEC_1900_TO_1970   0x83aa7e80

#define SNTP_LI_NO_WARNING 0
#define SNTP_VN_IPV4ONLY   3

#define SNTP_MODE_CLIENT   3
#define SNTP_MODE_SERVER   4

static inline void sntpmsg_clear( SNTPMessage* msg )
{
    memset( msg, 0, sizeof( SNTPMessage ) );
}

static inline void sntpmsg_set_li_vn_mode( SNTPMessage* msg, uint8_t li, uint8_t vn, uint8_t mode )
{
    msg->leapversionmode = ( uint8_t ) ( ( li << 6 ) | ( vn << 3 ) | mode);
}

static inline void sntptime_to_unixtime( struct timeval* unixtime, const SNTPTime* sntp )
{
    unixtime->tv_sec  = ntohl( sntp->second ) - SEC_1900_TO_1970;
    uint32_t fraction = ntohl( sntp->fraction );
    unixtime->tv_usec = ( double ) 1e6 * ( double ) fraction / ( double ) ( 1LL << 32 );
}

static inline void unixtime_to_sntptime( SNTPTime* sntp, const struct timeval* unixtime )
{
    sntp->second = htonl( unixtime->tv_sec + SEC_1900_TO_1970 );
    uint32_t fraction =  ( uint32_t ) ( 1e-6 * ( ( double ) (unixtime->tv_usec ) + 0.5f ) * ( double ) ( 1LL << 32 ) );
    sntp->fraction = htonl( fraction );
}


#endif
